/*
Dateiname		: queue.cpp
*/

#include "queue.h"
#include<iostream>
using namespace std;

/*Hauptfunktion fuer queue-Operationen
	Der Benutzer w�hlt durchzuf�hrende queue-Operationen aus, bis er die Beendigung des Programmes ausw�hlt
	
	Zu jeder Zeit werden angezeigt:
		- die letzte durchgefuehrte Operation (mit evtl aufgetretenen Fehlern)
		- die aktuelle queue
		- die Auswahl der zur Verfuegung stehenden Folgeoperationen*/
queue::queue(int gewaehlteGroesse)
{	
	size = gewaehlteGroesse;
	
	a = new int[size];
	initialisiereArray();

	head = 0;
	tail = 0;
}

queue::~queue()
{
	delete[] a;
}

//Alle Array-Elemente werden mit 0 initialisiert
void queue::initialisiereArray()
{
	for (int i = 0; i  < this->size; i++)
	{
		a[i] = 0;
	}
}

/*Ausgabe des Arrays
	Werte ausserhalb der head-tail-Grenzen: x
	Werte innerhalb der head-tail-Grenzen: Wert des Arrays an der jeweiligen Speicherstelle*/
void queue::ausgabeArray()
{
	//Anzahl darzustellender Array-stellen pro Zeile
	const int zeichenProZeile = 5;

	//Durchlauf des Arrays
	for (int i = 0; i < size; i++)
	{
		//head <= tail? (d.h. tail hat die Array-Grenze noch nicht erreicht)
		if (head <= tail)
		{
			//Aktuelle Arraystelle innerhlab der head-tail-Grenze?
			if ((i >= head) && (i < tail))
			{
				//Ausgabe des aktuellen Array-Wertes
				cout << a[i];
			}
			else
			{
				//Ausserhalb der head-tail-Grenze nur 'x' ausgeben
				cout << "x";
			}
		}
		//head > tail (d.h. tail hat die Array-Grenze ueberschritten und beginnt wieder bei 0)
		else
		{
			//Aktuelle Arraystelle ausserhalb der head-tail-Grenze?
			if ((i < head) && (i >= tail))
			{
				//Ausserhalb der head-tail-Grenze nur 'x' ausgeben
				cout << "x";
			}
			else
			{
				//Ausgabe des aktuellen Array-Wertes
				cout << a[i];
			}
		}
		
		//Einfuegen tabulator
		cout << "\t";

		//Wenn die passende Anzahl Array-Stellen ausgegeben wurde
		if ((i + 1) % zeichenProZeile == 0)
		{
			//Zeilenumbruch
			cout << endl;

			//Ausgabe von head und tail in dieser neuen Zeile
			ausgabeHeadTail(i);
		}
	}
	
	/*Wenn die Groesse des Arrays kein ganzzahliges Vielfaches der Variablen zeichenProZeile ist,
		wuerde fuer die letzte Array-Zeile keine Anzeige f�r head und tail gemacht werden.*/
	if ((size % zeichenProZeile) != 0)
	{
		//Zeilenumbruch
		cout << endl;
			
		/*Array wird zur head-tail-ausgabe um soviele Elemente erweitert,
			wie noetig um die arraygroesse zu einem
			ganzzahligen Vielfachen von zeichenProZeile zu machen*/
		ausgabeHeadTail((((size / 5) * 5) + 4));
	}
}

//Ausgabe von head und tail, falls in aktueller Zeile vorhanden
void queue::ausgabeHeadTail(int i)
{
	//Durchlauf des Arrays vom ersten bis zum letzten Element der aktuellen Zeile
	for (int j = ((((i + 1) / 5) - 1) * 5); j < (i + 1); j++)
	{
		//Aktuelles Element == head und tail?
		if ((j == head) && (j == tail))
		{
			//Ausgabe HT
			cout << "HT\t";
		}
		else
		{
			//Aktuelles Element == head ODER tail?
			if ((j == head) || (j == tail))
			{
				//Aktuelles Element == head
				if (j == head)
				{
					//Ausgabe H
					cout << "H\t";
				}
				//Aktuelles Element == tail
				if (j == tail)
				{
					//Ausgabe T
					cout << "T\t";
				}
			}
			else
			{
				//Ausgabe tabulator
				cout << "\t";
			}
		}
	}
	
	cout << endl;
}

/*Funktion zur Kontrolle ob die queue leer ist
	return true:	queue ist leer
	return false: queue ist nicht leer*/
bool queue::queueEmpty()
{
	bool empty = false;

	//Abfrage ob tail = head (Kritierium "leere queue")
	if (tail == head)
	{
		empty = true;
	}
	else
	{
		empty = false;
	}
	
	return empty;
}

/*Funktion zur Kontrolle ob die queue voll ist
	(d.h. alle Elemente bis auf eins belegt)
	return true:	queue ist voll
	return false: queue ist nicht voll*/
bool queue::queueFull()
{
	bool full = false;
	
	/*Hoerfuer sind 2 moeglcihe Faelle zu betrachten
		- head == 0
			--> wenn tail = size -1 ist, ist die queue voll
		- head != 0
			--> wenn tail == head -1 ist, ist die queue voll*/
	if (head == 0)
	{
		if (tail == size-1)
		{
			full = true;
		}
		else
		{
			full = false;
		}
	}
	else
	{
		if (tail == head - 1)
		{
			full = true;
		}
		else
		{
			full = false;
		}
	}
	
	return full;
}

/*Fuehrt die Operation enqueue auf die queue aus
	der einzuf�gende Wert wird in die Speicherstell geschrieben auf die tail zeigt
	und tail danach erh�ht, um auf das naechste freie Element zu zeigen*/
void queue::enqueue(int Wert)
{
	a[tail] = Wert;
	if (tail == size-1)
	{
		tail = 0;
	}
	else
	{
		tail++;
	}
}

/*Fuehrt die Operation dequeue auf die queue aus
	der Wert der Speicherstelle auf die head zeigt wird auf 0 gesetzt
	und head danach erhoeht*/
void queue::dequeue()
{
	a[head] = 0;
	if (head == size -1)
	{
		head = 0;
	}
	else
	{
		head++;
	}
}

//Loescht die bisherigen Inhalte der Shell
void queue::clearShell()
{
//	system("clear");
	cout << "\033[2J" << flush;
}

void queue::callEnqueue()
{
	cout << "Enqueue" << endl;
	cout << endl;

	//Abfrage ob queue voll ist
	if (queueFull() == false)
	//queue nicht voll --> einzufuegenden Wert eingeben --> Wert bei tail einfuegen --> tail erhoehen
	{
		cout << "Wert: ";
		int wert;
		cin >> wert;
		enqueue(wert);
		clearShell();
		cout << "--- Letzte durchgefuehrte Operation:" << endl;
		cout << "	Enqueue" << endl;
		cout << "	Wert: " << wert << endl;
		cout << endl;
	}
	else
	//queue ist voll --> Fehlerausgabe
	{
		clearShell();
		cout << "--- Letzte durchgefuehrte Operation:" << endl;
		cout << "	Enqueue" << endl;
		cout << endl;
		cout << "	Fehler: Queue voll" << endl;
	}
	
	cout << endl;
}

void queue::callDequeue()
{
	cout << "--- Letzte durchgefuehrte Operation:" << endl;
	cout << "	Dequeue" << endl;
	cout << endl;

	//abfrage ob queue leer ist
	if (queueEmpty() == false)
	//queue nicht leer --> head-Element loeschen --> head erhoehen
	{
		dequeue();
		cout <<endl;
	}
	else
	//queue leer --> Fehlerausgabe
	{
		cout << "	Fehler: Queue leer" << endl;
	}
}
